# ================================================================

use strict;

# ================================================================

our($iLibraryVersion)='20211225';

# ================================================================

our(%iUserAgentBrowserIconTable)=
(
	'Epiphany'				=> 'epiphany.png',
	'Google Chrome'				=> 'chrome.png',
	'Konqueror'				=> 'konqueror.png',
	'Lunascape'				=> 'lunascape.png',
	'Microsoft Internet Explorer ~10'	=> 'ie.png',
	'Microsoft Internet Explorer 11~'	=> 'ie.png',
	'Mozilla Firefox'			=> 'firefox.png',
	'Opera'					=> 'opera.png',
	'Safari'				=> 'safari.png',
	'Sleipnir'				=> 'sleipnir.png'
);

our(@iUserAgentBrowserTable)=
(
	['Googlebot/'		,'Googlebot'],
	['bingbot/'		,'bingbot'],
	['Yahoo! Slurp'		,'Yahoo! Slurp'],
	['Baiduspider/'		,'Baiduspider'],
	['YandexBot/'		,'YandexBot'],
	['Yeti/'		,'Yeti'],
	['compatible; MSIE '	,'Microsoft Internet Explorer ~10'],
	['Trident/'		,'Microsoft Internet Explorer 11~'],
	['Firefox/'		,'Mozilla Firefox'],
	['Edge/'		,'Microsoft Edge'],	# -- <!> contains "Chrome,Safari"
	['OPR/'			,'Opera'],		# -- <!> contains "Chrome,Safari"
	['Chrome/'		,'Google Chrome'],	# -- <!> contains "Safari"
	['Safari/'		,'Safari'],
	['Epiphany/'		,'Epiphany'],
	['Konqueror/'		,'Konqueror'],
	['Lunascape '		,'Lunascape'],
	['Lynx/'		,'Lynx'],
	['Sleipnir/'		,'Sleipnir'],
	['KDDI-'		,'au 3G Phone'],
	['SoftBank/'		,'SoftBank 3G Phone'],
	['Vodafone/'		,'Vodafone(SoftBank) 3G Phone'],
	['Wget/'		,'wget'],
	['curl/'		,'curl'],
	['libwww-perl/'		,'libwww-perl'],
);

# ================================================================

sub convertEUCJPToShiftJIS
{
	my($aString)=@_;

	return &convertXToY($aString,'euc-jp','shiftjis');
}

# ----------------------------------------------------------------

sub convertXToY
{
	my($aString,$aEncodingX,$aEncodingY)=@_;

	my($tResult)=$aString;

	use Encode;

	Encode::from_to($tResult,$aEncodingX,$aEncodingY);

	return $tResult;
}

# ----------------------------------------------------------------

sub decodeAsURL
{
	my($aTarget)=@_;

	my($tResult)=$aTarget;

	$tResult=~ s/%([0-9A-Fa-f][0-9A-Fa-f])/pack('C',hex($1))/eg;
	$tResult=~ s/\+/ /g;

	return $tResult;
}

# ----------------------------------------------------------------

sub deleteRecordIfMatch
{
	my($aDBFile,$aKey,$aValue,$aIsEncodedValue)=@_;

	if(!($aIsEncodedValue))
	{
		$aValue=&encodeAsURL($aValue);
	}

	my(@tFileData)=&readFileAsText($aDBFile);

	open(DATA_FILE,'>'.$aDBFile);
	flock(DATA_FILE,2);

	for(my($tCount1)=0;$tCount1<=($#tFileData);$tCount1++)
	{
		my(%tFileQuery)=&getQueryEntries($tFileData[$tCount1],1);

		if(&isDifferent($tFileQuery{$aKey},$aValue))
		{
			print DATA_FILE $tFileData[$tCount1];
		}
	}

	close(DATA_FILE);
}

# ----------------------------------------------------------------

sub encodeAsURL
{
	my($aTarget)=@_;

	my($tResult)=$aTarget;

	$tResult=~ s/ /\+/g;
	$tResult=~ s/(\W)/sprintf("%%%02X",unpack('C',$1))/eg;

	return $tResult;
}

# ----------------------------------------------------------------

sub getDomainURL
{
	my($tResult)='';

	if(&isSame($ENV{'HTTPS'},'on'))
	{
		$tResult='https://';
	}
	else
	{
		$tResult='http://';
	}

	$tResult.=$ENV{'HTTP_HOST'};

	if(&isDifferent(($ENV{'SERVER_PORT'},'80'))&&(&isDifferent($ENV{'SERVER_PORT'},'443')))
	{
		$tResult.=':'.$ENV{'SERVER_PORT'};
	}

	return $tResult;
}

# ----------------------------------------------------------------

sub getDumpIfExists
{
	my($aTarget)=@_;

	my($tResult)='';

	if(-e $aTarget)
	{
		my(@tDumpData)=&readFileAsText($aTarget);

		for(my($tCount1)=0;$tCount1<=($#tDumpData);$tCount1++)
		{
			$tResult.=$tDumpData[$tCount1];
		}
	}

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLAboutThisSoftware
{
	my($aApplicationName,$aApplicationVersion,$aLibraryVersion)=@_;

	my($tResult)='';

	$tResult.=&getHTMLCaption('ΥեȥˤĤ',1);
	$tResult.=&getHTMLParagraphOpen();
	$tResult.=$aApplicationName.'<br>'."\n";
	$tResult.='version '.$aApplicationVersion.' (+Library:'.$aLibraryVersion.')<br>'."\n";
	$tResult.=&getHTMLBreak();
	$tResult.='Programmed by Kei Kishiro'."\n";
	$tResult.=&getHTMLParagraphClose();

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLBreak
{
	my($tResult)='';

	$tResult.='<br>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLCaption
{
	my($aCaption,$aSize)=@_;

	my($tResult)='';

	$tResult.='<h'.$aSize.'>'.$aCaption.'</h'.$aSize.'>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLCheckBox
{
	my($aName,$aValue,$aIsChecked)=@_;

	my($tResult)='';

	if($aIsChecked)
	{
		$tResult.='<input type="checkbox" name="'.$aName.'" value="1" checked>'.$aValue.'<br>';
	}
	else
	{
		$tResult.='<input type="checkbox" name="'.$aName.'" value="1">'.$aValue.'<br>';
	}

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLDeleteCompleted
{
	my($tResult)='';

	$tResult.=&getHTMLCaption('δλ',1);
	$tResult.=&getHTMLParagraphOpen();
	$tResult.='Υ֥Ȥޤ'."\n";
	$tResult.=&getHTMLParagraphClose();

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLDocumentClose
{
	my($tResult)='';

	$tResult.='</div>'."\n";
	$tResult.='</body>'."\n";
	$tResult.='</html>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLDocumentCloseWithApplicationVersion
{
	my($aApplicationName,$aApplicationVersion,$aLibraryVersion)=@_;

	my($tResult)='';

	$tResult.=&getHTMLLine();
	$tResult.='<address>'.$aApplicationName.' version '.$aApplicationVersion.' (Library:'.$aLibraryVersion.') / Programmed by Kei Kishiro</address>'."\n";
	$tResult.='</div>'."\n";
	$tResult.='</body>'."\n";
	$tResult.='</html>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLDocumentOpen
{
	my($aTitle,$aContentLanguage,$aCharset,$aCSSFiles,$aJavaScripts)=@_;

	my($tResult)='';

	$tResult.='<!doctype html public "-//W3C//DTD HTML 4.01 Transitional//EN">'."\n";
	$tResult.='<html lang="'.$aContentLanguage.'">'."\n";
	$tResult.='<head>'."\n";
	$tResult.='<meta name="viewport" content="width=device-width,initial-scale=1">'."\n";
	$tResult.='<meta name="ROBOTS" content="NOARCHIVE">'."\n";
	$tResult.='<meta http-equiv="Content-Type" content="text/html; charset='.$aCharset.'">'."\n";

	if(($#$aCSSFiles)>=0)
	{
		$tResult.='<style type="text/css">'."\n";

		for(my($tCount1)=0;$tCount1<=($#$aCSSFiles);$tCount1++)
		{
			$tResult.=&getDumpIfExists($$aCSSFiles[$tCount1]);
		}

		$tResult.='</style>'."\n";
	}

	if(($#$aJavaScripts)>=0)
	{
		$tResult.='<script type="text/javascript">'."\n";
		$tResult.='<!--'."\n";

		for(my($tCount1)=0;$tCount1<=($#$aJavaScripts);$tCount1++)
		{
			$tResult.=&getDumpIfExists($$aJavaScripts[$tCount1]);
		}

		$tResult.='//-->'."\n";
		$tResult.='</script>'."\n";
	}

	$tResult.='<title>'.$aTitle.'</title>'."\n";
	$tResult.='</head>'."\n";
	$tResult.='<body>'."\n";
	$tResult.='<div id="Outline">'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLFormClose
{
	my($tResult)='';

	$tResult.='</form>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLFormOpen
{
	my($aAction,$aMethod,$aEncodeType)=@_;

	my($tResult)='';

	my($tEncodeType)='';

	if(&isSame($aEncodeType,''))
	{
		$tEncodeType='application/x-www-form-urlencoded';
	}
	else
	{
		$tEncodeType=$aEncodeType;
	}

	$tResult.='<form action="'.$aAction.'" method="'.$aMethod.'" enctype="'.$tEncodeType.'">'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLHiddenField
{
	my($aName,$aValue)=@_;

	my($tResult)='';

	$tResult.='<input type="hidden" name="'.$aName.'" value="'.$aValue.'">'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLLine
{
	my($tResult)='';

	$tResult.='<hr>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLMetaRefresh
{
	my($aURL,$aInterval)=@_;

	my($tResult)='';

	$tResult.='<meta http-equiv="refresh" content="'.$aInterval.';URL='.$aURL.'">'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLParagraphClose
{
	my($tResult)='';

	$tResult.='</p>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLParagraphOpen
{
	my($tResult)='';

	$tResult.='<p>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLSubmitButtons
{
	my($aNameSubmit,$aNameReset)=@_;

	my($tResult)='';

	$tResult.='<input type="submit" value="'.$aNameSubmit.'">';

	if($aNameReset ne '')
	{
		$tResult.=' <input type="reset" value="'.$aNameReset.'">';
	}

	$tResult.='<br>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLTextBox
{
	my($aName,$aValue,$aColSize,$aRowSize)=@_;

	my($tResult)='';

	$tResult.='<textarea ';

	if(&isDifferent($aName,''))
	{
			$tResult.='name="'.$aName.'"';
	}

	if(&isDifferent($aColSize,''))
	{
			$tResult.='cols="'.$aColSize.'"';
	}

	if(&isDifferent($aRowSize,''))
	{
			$tResult.='rows="'.$aRowSize.'"';
	}

	$tResult.='">';

	if(&isDifferent($aValue,''))
	{
			$tResult.=$aValue;
	}

	$tResult.='</textarea><br>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLUnknownAction
{
	my($tResult)='';

	$tResult.=&getHTMLCaption('顼ˤ',1);
	$tResult.=&getHTMLParagraphOpen();
	$tResult.='ꥯȤ̵뤵ޤ<br>'."\n";
	$tResult.='¸ߤʤ󤬻ꤵޤλְ꤬äƤޤ'."\n";
	$tResult.=&getHTMLParagraphClose();

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLUpdateCompleted
{
	my($tResult)='';

	$tResult.=&getHTMLCaption('δλ',1);
	$tResult.=&getHTMLParagraphOpen();
	$tResult.='Υ֥Ȥޤ'."\n";
	$tResult.=&getHTMLParagraphClose();

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTTPContentHeader
{
	my($aContentLanguage,$aCharset)=@_;

	my($tResult)='';

	$tResult.='Content-type: text/html; charset='.$aCharset."\n";
	$tResult.='Content-Language: '.$aContentLanguage."\n";
	$tResult.='Cache-Control: no-cache'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTTPResponse200Header
{
	my($tResult)='';

	$tResult.='HTTP/1.1 200 OK'."\n";
	$tResult.='Server: N/A'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getPostQueryEntries
{
	my($aDoDecode)=@_;

	my(%tReturn)=();

	if(&isSame($ENV{'REQUEST_METHOD'},'POST'))
	{
		read(STDIN,my($tPostString),$ENV{'CONTENT_LENGTH'});
		%tReturn=&getQueryEntries($tPostString,$aDoDecode);
	}

	return %tReturn;
}

# ----------------------------------------------------------------

sub getQueryEntries
{
	my($aString,$aDoDecode)=@_;

	my(@tQuery)=split(/\&/,$aString);
	my(%tReturn)=();

	for(my($tCount1)=0;$tCount1<=($#tQuery);$tCount1++)
	{
		my($tName,$tValue)=split(/=/,$tQuery[$tCount1]);
		chomp($tValue);

		if($aDoDecode)
		{
			$tReturn{$tName}=&decodeAsURL($tValue);
		}
		else
		{
			$tReturn{$tName}=$tValue;
		}
	}

	return %tReturn;
}

# ----------------------------------------------------------------

sub getRecordIfMatch
{
	my($aDBFile,$aKey,$aValue,$aDoDecode)=@_;

	my(@tResult)=();

	my(@tFileData)=&readFileAsText($aDBFile);

	for(my($tCount1)=0;$tCount1<=($#tFileData);$tCount1++)
	{
		my(%tCurrentRecord)=&getQueryEntries($tFileData[$tCount1],$aDoDecode);

		if(&isSame($tCurrentRecord{$aKey},$aValue))
		{
			push(@tResult,\%tCurrentRecord);
		}
	}

	return @tResult;
}

# ----------------------------------------------------------------

sub getScriptURL
{
	return &getDomainURL().$ENV{'SCRIPT_NAME'};
}

# ----------------------------------------------------------------

sub getURLQueryEntries
{
	my($aDoDecode)=@_;

	return &getQueryEntries($ENV{'QUERY_STRING'},$aDoDecode);
}

# ----------------------------------------------------------------

sub getUserAgentIconURL
{
	my($aUserAgent,$aBaseURL)=@_;

	my($tFileName)='';

	$tFileName=$main::iUserAgentBrowserIconTable{&identifyBrowser($aUserAgent)};

	if(&isSame($tFileName,''))
	{
		$tFileName='unknown.png';
	}

	return $aBaseURL.$tFileName;
}

# ----------------------------------------------------------------

sub identifyBrowser
{
	my($aUserAgent)=@_;

	return &identifyName($aUserAgent,\@main::iUserAgentBrowserTable);
}

# ----------------------------------------------------------------

sub identifyName
{
	my($aString,$aTable)=@_;

	my($tResult)='';

	for(my($tCount1)=0;$tCount1<=($#$aTable);$tCount1++)
	{
		if(&isContained($aString,$$aTable[$tCount1][0]))
		{
			$tResult=$$aTable[$tCount1][1];

			last;
		}
	}

	return $tResult;
}

# ----------------------------------------------------------------

sub isContained
{
	my($aString1,$aString2)=@_;

	if(index($aString1,$aString2)>=0)
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

# ----------------------------------------------------------------

sub isDifferent
{
	my($aString1,$aString2)=@_;

	return ($aString1 ne $aString2);
}

# ----------------------------------------------------------------

sub isSame
{
	my($aString1,$aString2)=@_;

	return ($aString1 eq $aString2);
}

# ----------------------------------------------------------------

sub printResponseCache
{
	my($aHeader,$aBody)=@_;

	if((&isDifferent($aHeader,''))&&(&isDifferent($aBody,'')))
	{
		print $aHeader;
		print 'Content-Length: '.length($aBody)."\n\n";
		print $aBody;
	}
}

# ----------------------------------------------------------------

sub readFileAsText
{
	my($aTarget)=@_;

	open(DATA_FILE,'<'.$aTarget);
	flock(DATA_FILE,1);
	my(@tFileData)=<DATA_FILE>;
	close(DATA_FILE);

	return @tFileData;
}

# ----------------------------------------------------------------

sub sanitize
{
	my($aTarget)=@_;

	my($tResult)=$aTarget;

	$tResult=~ s/\&/&amp;/g;
	$tResult=~ s/</&lt;/g;
	$tResult=~ s/>/&gt;/g;
	$tResult=~ s/"/&quot;/g;

	return $tResult;
}

# ----------------------------------------------------------------

sub updateRecordIfMatch
{
	my($aDBFile,$aKey,$aValue,$aAssociativeArray,$aIsEncodedValue)=@_;

	if(!($aIsEncodedValue))
	{
		$aValue=&encodeAsURL($aValue);
	}

	my(@tFileData)=&readFileAsText($aDBFile);

	open(DATA_FILE,'>'.$aDBFile);
	flock(DATA_FILE,2);

	for(my($tCount1)=0;$tCount1<=($#tFileData);$tCount1++)
	{
		my(%tFileQuery)=&getQueryEntries($tFileData[$tCount1],0);

		foreach(sort keys %tFileQuery)
		{
			my($tKey)=$_;
			my($tValue)=$tFileQuery{$tKey};
		}

		if(&isSame($tFileQuery{$aKey},$aValue))
		{
			foreach(sort keys %$aAssociativeArray)
			{
				my($tKey)=$_;
				my($tValue)=$$aAssociativeArray{$tKey};

				if($aIsEncodedValue)
				{
					$tFileQuery{$tKey}=$tValue;
				}
				else
				{
					$tFileQuery{$tKey}=&encodeAsURL($tValue);
				}
			}

			my($tRecordLine)='';

			foreach(sort keys %tFileQuery)
			{
				my($tKey)=$_;
				my($tValue)=$tFileQuery{$tKey};

				if(&isDifferent($tValue,''))
				{
					if(&isDifferent($tRecordLine,''))
					{
						$tRecordLine.='&';
					}

					$tRecordLine.=$tKey.'='.$tValue;
				}
			}

			print DATA_FILE $tRecordLine."\n";
		}
		else
		{
			print DATA_FILE $tFileData[$tCount1];
		}
	}

	close(DATA_FILE);
}

# ================================================================

1;
